// Copyright 2017-2022 @polkadot/typegen authors & contributors
// SPDX-License-Identifier: Apache-2.0
import path from 'path';
import yargs from 'yargs';
import { formatNumber } from '@polkadot/util';
import { WebSocket } from '@polkadot/x-ws';
import { generateDefaultConsts, generateDefaultErrors, generateDefaultEvents, generateDefaultQuery, generateDefaultRpc, generateDefaultTx } from "./generate/index.js";
import { HEADER, writeFile } from "./util/index.js";

function generate(metaHex, pkg, output, isStrict) {
  console.log(`Generating from metadata, ${formatNumber((metaHex.length - 2) / 2)} bytes`);
  const base = path.join(process.cwd(), output);
  const extraTypes = pkg // eslint-disable-next-line @typescript-eslint/no-var-requires
  ? {
    [pkg]: require(path.join(base, 'definitions'))
  } : {};
  let customLookupDefinitions;

  try {
    customLookupDefinitions = {
      rpc: {},
      // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-assignment, @typescript-eslint/no-var-requires
      types: require(path.join(base, 'lookup.ts')).default
    };
  } catch (error) {
    console.log('No custom definitions found.');
  }

  generateDefaultConsts(path.join(base, 'augment-api-consts.ts'), metaHex, extraTypes, isStrict, customLookupDefinitions);
  generateDefaultErrors(path.join(base, 'augment-api-errors.ts'), metaHex, extraTypes, isStrict);
  generateDefaultEvents(path.join(base, 'augment-api-events.ts'), metaHex, extraTypes, isStrict, customLookupDefinitions);
  generateDefaultQuery(path.join(base, 'augment-api-query.ts'), metaHex, extraTypes, isStrict, customLookupDefinitions);
  generateDefaultRpc(path.join(base, 'augment-api-rpc.ts'), extraTypes);
  generateDefaultTx(path.join(base, 'augment-api-tx.ts'), metaHex, extraTypes, isStrict, customLookupDefinitions);
  writeFile(path.join(base, 'augment-api.ts'), () => [HEADER('chain'), ...[...['consts', 'errors', 'events', 'query', 'tx', 'rpc'].filter(key => !!key).map(key => `./augment-api-${key}`)].map(path => `import '${path}';\n`)].join(''));
  process.exit(0);
}

export function main() {
  const {
    endpoint,
    output,
    package: pkg,
    strict: isStrict
  } = yargs.strict().options({
    endpoint: {
      description: 'The endpoint to connect to (e.g. wss://kusama-rpc.polkadot.io) or relative path to a file containing the JSON output of an RPC state_getMetadata call',
      required: true,
      type: 'string'
    },
    output: {
      description: 'The target directory to write the data to',
      required: true,
      type: 'string'
    },
    package: {
      description: 'Optional package in output location (for extra definitions)',
      type: 'string'
    },
    strict: {
      description: 'Turns on strict mode, no output of catch-all generic versions',
      type: 'boolean'
    }
  }).argv;

  if (endpoint.startsWith('wss://') || endpoint.startsWith('ws://')) {
    try {
      const websocket = new WebSocket(endpoint);

      websocket.onclose = event => {
        console.error(`disconnected, code: '${event.code}' reason: '${event.reason}'`);
        process.exit(1);
      };

      websocket.onerror = event => {
        console.error(event);
        process.exit(1);
      };

      websocket.onopen = () => {
        console.log('connected');
        websocket.send('{"id":"1","jsonrpc":"2.0","method":"state_getMetadata","params":[]}');
      };

      websocket.onmessage = message => {
        generate(JSON.parse(message.data).result, pkg, output, isStrict);
      };
    } catch (error) {
      process.exit(1);
    }
  } else {
    // eslint-disable-next-line @typescript-eslint/no-var-requires
    generate(require(path.join(process.cwd(), endpoint)).result, pkg, output, isStrict);
  }
}